package speiger.src.builder.processor;

import java.io.BufferedWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Set;

import speiger.src.builder.base.Template;
import speiger.src.builder.mappers.IMapper;

public class BuildTask implements Runnable
{
	Path basePath;
	Template template;
	TemplateProcess process;
	Set<IMapper>[] mappers;
	boolean silencedSuccess;
	
	public BuildTask(Path basePath, Template template, TemplateProcess process, boolean silencedSuccess, Set<IMapper>[] mappers)
	{
		this.basePath = basePath;
		this.template = template;
		this.process = process;
		this.silencedSuccess = silencedSuccess;
		this.mappers = mappers;
	}
	
	@Override
	public void run()
	{
		String s = template.build(new CompiledArguments(process), mappers != null, mappers != null ? mappers[1] : null);
		if(mappers != null)
		{
			mappers[0].addAll(process.mappers);
		}
		Path path = (process.pathBuilder != null ? process.pathBuilder.apply(basePath) : basePath).resolve(process.fileName);
		try
		{
			Files.createDirectories(path.getParent());			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		try(BufferedWriter writer = Files.newBufferedWriter(path))
		{
			writer.write(s);
			writer.flush();
			if(!silencedSuccess) System.out.println("Created: "+process.fileName);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
