package speiger.src.builder.processor;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.UnaryOperator;

import speiger.src.builder.mappers.IMapper;

public class TemplateProcess
{
	UnaryOperator<Path> pathBuilder;
	String fileName;
	Set<String> parsePool = new HashSet<>();
	Map<String, Integer> parseValues = new HashMap<>();
	List<IMapper> mappers = new ArrayList<>();
	
	public TemplateProcess(String fileName)
	{
		this.fileName = fileName;
	}
	
	public TemplateProcess setPathBuilder(UnaryOperator<Path> pathBuilder)
	{
		this.pathBuilder = pathBuilder;
		return this;
	}
	
	public TemplateProcess addFlags(String...flags)
	{
		parsePool.addAll(Arrays.asList(flags));
		return this;
	}
	
	public TemplateProcess addFlags(Collection<String> flags)
	{
		parsePool.addAll(flags);
		return this;
	}
	
	public TemplateProcess addValue(String key, int value)
	{
		parseValues.put(key, value);
		return this;
	}
	
	public TemplateProcess addValues(Map<String, Integer> values)
	{
		parseValues.putAll(values);
		return this;
	}
	
	public TemplateProcess addMapper(IMapper mapper)
	{
		mappers.add(mapper);
		return this;
	}
	
	public TemplateProcess addMappers(Collection<IMapper> mappers)
	{
		this.mappers.addAll(mappers);
		return this;
	}
}
